@@ -6,7 +6,7 @@ import sys
 
 from . import __version__
 from .allkeys import loadextrabl, urllookup
-from .checks import (allchecks, checkcrt, checkpubkey, checkrsa, checksshpubkey, defaultchecks,
+from .checks import (_checkkey, allchecks, checkcrt, checkrsa, checksshpubkey, defaultchecks,
                      detectandcheck)
 from .dkim import parsedkim
 from .dnssec import checkdnskey
@@ -23,6 +23,14 @@ count = 0
 PRECRT = "-----BEGIN CERTIFICATE-----\n"
 POSTCRT = "\n-----END CERTIFICATE-----\n"
 
+_parseerrmsg = {
+    "privleak": "Private instead of public key",
+    "keyparseerror": "Error decoding DER/ASN.1 key structure",
+    "expectedrsa": "Wrong key type, expected RSA key",
+    "wrongkeylength": "Wrong key length",
+    "unknowntype": "Unknown key type",
+}
+
 
 def _sighandler(_signum, _handler):
     print(f"{count} keys processed", file=sys.stderr)
@@ -31,14 +39,17 @@ def _sighandler(_signum, _handler):
 def _printresults(key, where, args):
     if args.json:
         jout = key
-        for val in ["n", "e", "x"]:
+        jout["where"] = where
+        for val in ["n", "e", "x", "y"]:
             if val in jout:
                 jout[val] = f"{jout[val]:x}"
         for result in jout["results"]:
-            if "p" in jout["results"][result]:
-                jout["results"][result]["p"] = f'{jout["results"][result]["p"]:x}'
-                jout["results"][result]["q"] = f'{jout["results"][result]["q"]:x}'
-        print(json.dumps(jout))
+            for val in ["p", "q", "e"]:
+                if val in jout["results"][result]:
+                    jout["results"][result][val] = f'{jout["results"][result][val]:x}'
+        if args.all or jout["results"] or \
+           key["type"] in ["unsupported", "unparseable", "notfound"]:
+            print(json.dumps(jout))
         return
     kn = key["type"]
     if "bits" in key:
@@ -48,7 +59,12 @@ def _printresults(key, where, args):
     if key["type"] == "unsupported":
         _warnmsg(f"Unsupported key type, {_esc(where)}")
     elif key["type"] == "unparseable":
-        _warnmsg(f"Unparseable input, {_esc(where)}")
+        reason = ""
+        if reason in key and key["reason"] in _parseerrmsg:
+            reason = f" ({_parseerrmsg[key['reason']]})"
+        elif "reason" in key:
+            reason = f" ({key['reason']})"
+        _warnmsg(f"Unparseable input{reason}, {_esc(where)}")
     elif key["type"] == "notfound":
         _warnmsg(f"No key found, {_esc(where)}")
     elif args.verbose or args.all:
@@ -82,88 +98,48 @@ def runcli():
         pass
 
     ap = argparse.ArgumentParser()
-    ap.add_argument(
-        "infiles", nargs="*", help="Input file (certificate, csr or public key)"
-    )
-    ap.add_argument(
-        "-c", "--checks", help="Comma-separated list of checks (default: all)"
-    )
+    ap.add_argument("infiles", nargs="*", help="Input file (certificate, csr or public key)")
+    ap.add_argument("-c", "--checks", help="Comma-separated list of checks (default: all)")
     ap.add_argument("--list", action="store_true", help="Show list of possible checks")
-    ap.add_argument(
-        "-m",
-        "--moduli",
-        action="store_true",
-        help="Input file is list of RSA hex moduli",
-    )
-    ap.add_argument(
-        "--crt-lines", action="store_true", help="Input file is list of base64 certs"
-    )
-    ap.add_argument(
-        "--ssh-lines", action="store_true", help="Input file is list of ssh public keys"
-    )
+    ap.add_argument("-m", "--moduli", action="store_true",
+                    help="Input file is list of RSA hex moduli")
+    ap.add_argument("--crt-lines", action="store_true", help="Input file is list of base64 certs")
+    ap.add_argument("--ssh-lines", action="store_true",
+                    help="Input file is list of ssh public keys")
     ap.add_argument("--dkim", action="store_true", help="Scan DKIM records (in files)")
-    ap.add_argument(
-        "--dkim-dns",
-        action="store_true",
-        help="Scan DKIM DNS record (hostnames instead of files)",
-    )
+    ap.add_argument("--dkim-dns", action="store_true",
+                    help="Scan DKIM DNS record (hostnames instead of files)")
     ap.add_argument("--dnssec", action="store_true", help="Scan DNSKEY/DNSSEC records (in files)")
     ap.add_argument("--jwk", action="store_true", help="Scan JSON Web Keys / Key Sets")
     ap.add_argument("-a", "--all", action="store_true", help="Show all keys")
-    ap.add_argument(
-        "-w",
-        "--warnings",
-        action="store_true",
-        help="Enable extra warnings (keysize etc.)",
-    )
+    ap.add_argument("-w", "--warnings", action="store_true",
+                    help="Enable extra warnings (keysize etc.)")
     ap.add_argument("-v", "--verbose", action="store_true", help="Verbose output")
     ap.add_argument("-j", "--json", action="store_true", help="JSON output")
-    ap.add_argument(
-        "-q", "--quiet", action="store_true", help="Quiet output (for update commands)"
-    )
-    ap.add_argument(
-        "-u", "--url", action="store_true", help="Show private key URL if possible"
-    )
+    ap.add_argument("-q", "--quiet", action="store_true", help="Quiet output (for update commands)")
+    ap.add_argument("-u", "--url", action="store_true", help="Show private key URL if possible")
     ap.add_argument("--update-bl", action="store_true", help="Update blocklist")
-    ap.add_argument(
-        "--update-bl-and-urls",
-        action="store_true",
-        help="Update blocklist and optional URL lookup list",
-    )
+    ap.add_argument("-U", "--update-bl-and-urls", action="store_true",
+                    help="Update blocklist and optional URL lookup list")
     ap.add_argument("--extrabl", help="comma-separated list of extra blocklist files")
-    ap.add_argument(
-        "-t",
-        "--tls",
-        action="store_true",
-        help="Scan TLS (pass hostnames or IPs instead of files)",
-    )
+    ap.add_argument("-t", "--tls", action="store_true",
+                    help="Scan TLS (pass hostnames or IPs instead of files)")
     # default ports for https, smtps, imaps, pop3s, ldaps, ftps
     # and 8443 as a common non-default https port
-    ap.add_argument(
-        "--tls-ports",
-        default="443,465,636,990,993,995,8443",
-        help="TLS ports (comma-separated)",
-    )
-    ap.add_argument(
-        "-s",
-        "--ssh",
-        action="store_true",
-        help="Scan SSH (pass hostnames or IPs instead of files)",
-    )
+    ap.add_argument("--tls-ports", default="443,465,636,990,993,995,8443",
+                    help="TLS ports (comma-separated)")
+    ap.add_argument("-s", "--ssh", action="store_true",
+                    help="Scan SSH (pass hostnames or IPs instead of files)")
     ap.add_argument("--ssh-ports", default="22", help="SSH ports (comma-separated)")
     ap.add_argument("--version", action="version", version=__version__)
     args = ap.parse_args()
 
-    if (
-        (args.moduli and args.crt_lines)
-        or (args.moduli and args.ssh_lines)
-        or (args.ssh_lines and args.crt_lines)
-    ):
+    if (args.moduli and args.crt_lines) or (args.moduli and args.ssh_lines) \
+       or (args.ssh_lines and args.crt_lines):
         _errexit("Multiple input format parameters cannot be combined.")
 
-    if (args.moduli or args.crt_lines or args.ssh_lines) and (
-        args.tls or args.ssh or args.dkim_dns
-    ):
+    if (args.moduli or args.crt_lines or args.ssh_lines) and \
+       (args.tls or args.ssh or args.dkim_dns):
         _errexit("Scan modes and input file modes cannot be combined.")
 
     if args.update_bl_and_urls:
@@ -228,10 +204,15 @@ def runcli():
             for record in records.answer[-1]:
                 dk = b"".join(record.strings).decode()
                 key = parsedkim(dk)
-                if key:
-                    r = checkpubkey(key, checks=userchecks)
-                    _printresults(r, host, args)
-                    found = True
+
+                if not key:
+                    continue
+                found = True
+                if isinstance(key, str):
+                    r = {"type": "unparseable", "reason": key, "results": {}}
+                else:
+                    r = _checkkey(key, checks=userchecks)
+                _printresults(r, host, args)
             if not found:
                 _warnmsg(f"No DKIM/DomainKeys key in TXT record, {_esc(host)}")
 
@@ -264,9 +245,12 @@ def runcli():
             f = sys.stdin
         else:
             f = open(fn, errors="replace")
+
+        if args.moduli or args.ssh_lines or args.crt_lines or args.dkim:
+            lcount = 0
+
         if args.moduli:
             for line in f:
-                count += 1
                 try:
                     if line.startswith("Modulus="):
                         n = int(line[8:], 16)
@@ -283,9 +267,12 @@ def runcli():
                 r = {"type": "rsa", "bits": n.bit_length()}
                 r["results"] = checkrsa(n, checks=userchecks)
                 _printresults(r, f"modulus {n:02x}", args)
+                count += 1
+                lcount += 1
         elif args.crt_lines:
             lno = 0
             for line in f:
+                lno += 1
                 desc = f"{fn}[{lno}]"
                 ll = re.split("[,; ]", line.rstrip(), maxsplit=1)
                 if len(ll) == 2:
@@ -293,29 +280,46 @@ def runcli():
                 crt = PRECRT + ll[0] + POSTCRT
                 r = checkcrt(crt, checks=userchecks)
                 _printresults(r, desc, args)
-                lno += 1
                 count += 1
+                lcount += 1
         elif args.ssh_lines:
             lno = 0
-            for line in f:
+            for sline in f:
+                lno += 1
+                line = sline.strip()
+                if line == "" or line.startswith("#"):
+                    continue
                 desc = f"{fn}[{lno}]"
-                ll = line.rstrip().split(" ", 2)
+                if not line.startswith(("ssh-", "ecdsa-")):
+                    if "ssh-" in line:
+                        line = "ssh-" + line.split("ssh-", 1)[1]
+                    elif "ecdsa-" in line:
+                        line = "ecdsa-" + line.split("ecdsa-", 1)[1]
+                    else:
+                        _warnmsg(f"No SSH key in {desc}")
+                        continue
+                ll = line.split(" ", 2)
                 if len(ll) == 3:
                     desc += f" {ll[2]}"
                 r = checksshpubkey(line, checks=userchecks)
                 _printresults(r, desc, args)
-                lno += 1
                 count += 1
+                lcount += 1
         elif args.dkim:
             lno = 0
             for line in f:
+                lno += 1
                 desc = f"{fn}[{lno}]"
                 key = parsedkim(line)
-                if key:
-                    r = checkpubkey(key, checks=userchecks)
-                    _printresults(r, desc, args)
-                    count += 1
-                lno += 1
+                if not key:
+                    continue
+                if isinstance(key, str):
+                    r = {"type": "unparseable", "reason": key, "results": {}}
+                else:
+                    r = _checkkey(key, checks=userchecks)
+                _printresults(r, desc, args)
+                count += 1
+                lcount += 1
         elif args.dnssec:
             fcontent = f.read(MAXINPUTSIZE)
 
@@ -331,6 +335,10 @@ def runcli():
             r = detectandcheck(fcontent, checks=userchecks)
             _printresults(r, fn, args)
 
+        if args.moduli or args.ssh_lines or args.crt_lines or args.dkim:
+            if lcount == 0:
+                _printresults({"type": "notfound", "results": {}}, fn, args)
+
         if fn != "-":
             f.close()
 
